@@ -43,7 +43,8 @@ pub use options::{FieldExtension, HashFunction, ProofOptions};
 
 mod air;
 pub use air::{
-    Air, AirContext, Assertion, BoundaryConstraint, BoundaryConstraintGroup,
-    ConstraintCompositionCoefficients, ConstraintDivisor, DeepCompositionCoefficients,
-    EvaluationFrame, TraceInfo, TransitionConstraintDegree, TransitionConstraintGroup,
+    Air, AirContext, Assertion, AuxTraceRandElements, BoundaryConstraint, BoundaryConstraintGroup,
+    BoundaryConstraints, ConstraintCompositionCoefficients, ConstraintDivisor,
+    DeepCompositionCoefficients, EvaluationFrame, TraceInfo, TraceLayout,
+    TransitionConstraintDegree, TransitionConstraintGroup, TransitionConstraints,
 };
