@@ -10,6 +10,11 @@ use utils::{
     SliceReader,
 };
 
+// TYPE ALIASES
+// ================================================================================================
+
+type ParsedOodFrame<E> = (EvaluationFrame<E>, Option<EvaluationFrame<E>>, Vec<E>);
+
 // OUT-OF-DOMAIN FRAME
 // ================================================================================================
 /// Trace and constraint polynomial evaluations at an out-of-domain point.
@@ -25,44 +30,26 @@ use utils::{
 /// evaluations, [parse()](OodFrame::parse) function should be used.
 #[derive(Clone, Debug, Default, Eq, PartialEq)]
 pub struct OodFrame {
-    trace_at_z1: Vec<u8>,
-    trace_at_z2: Vec<u8>,
+    trace_states: Vec<u8>,
     evaluations: Vec<u8>,
 }
 
 impl OodFrame {
-    // CONSTRUCTOR
-    // --------------------------------------------------------------------------------------------
-    /// Returns a new [OodFrame] instantiated with the provided evaluation frame and a vector of
-    /// out-of-domain constraint evaluations.
-    ///
-    /// # Panics
-    /// Panics if `evaluations` is an empty vector.
-    pub fn new<E: FieldElement>(frame: EvaluationFrame<E>, evaluations: Vec<E>) -> Self {
-        let mut result = Self::default();
-        result.set_evaluation_frame(&frame);
-        result.set_constraint_evaluations(&evaluations);
-        result
-    }
-
     // UPDATERS
     // --------------------------------------------------------------------------------------------
 
-    /// Updates evaluation frame portion of this out-of-domain frame.
+    /// Updates the trace state portion of this out-of-domain frame.
     ///
     /// # Panics
     /// Panics if evaluation frame has already been set.
-    pub fn set_evaluation_frame<E: FieldElement>(&mut self, frame: &EvaluationFrame<E>) {
+    pub fn set_trace_states<E: FieldElement>(&mut self, trace_states: &[Vec<E>]) {
         assert!(
-            self.trace_at_z1.is_empty(),
-            "evaluation frame has already been set"
+            self.trace_states.is_empty(),
+            "trace sates have already been set"
         );
-        assert!(
-            self.trace_at_z2.is_empty(),
-            "evaluation frame has already been set"
-        );
-        frame.current().write_into(&mut self.trace_at_z1);
-        frame.next().write_into(&mut self.trace_at_z2);
+        for trace_state in trace_states {
+            trace_state.write_into(&mut self.trace_states);
+        }
     }
 
     /// Updates constraint evaluation portion of this out-of-domain frame.
@@ -85,56 +72,63 @@ impl OodFrame {
 
     // PARSER
     // --------------------------------------------------------------------------------------------
-    /// Returns an evaluation frame and a vector of out-of-domain constraint evaluations contained
-    /// in `self`.
+    /// Returns main and auxiliary (if any) trace evaluation frames and a vector of out-of-domain
+    /// constraint evaluations contained in `self`.
     ///
     /// # Panics
-    /// Panics if either `trace_width` or `num_evaluations` are equal to zero.
+    /// Panics if either `main_trace_width` or `num_evaluations` are equal to zero.
     ///
     /// # Errors
     /// Returns an error if:
-    /// * A valid [EvaluationFrame] for the specified `trace_width` could not be parsed from the
-    ///   internal bytes.
+    /// * Valid [EvaluationFrame]s for the specified `main_trace_width` and `aux_trace_width`
+    ///   could not be parsed from the internal bytes.
     /// * A vector of evaluations specified by `num_evaluations` could not be parsed from the
     ///   internal bytes.
     /// * Any unconsumed bytes remained after the parsing was complete.
     pub fn parse<E: FieldElement>(
         self,
-        trace_width: usize,
+        main_trace_width: usize,
+        aux_trace_width: usize,
         num_evaluations: usize,
-    ) -> Result<(EvaluationFrame<E>, Vec<E>), DeserializationError> {
-        assert!(trace_width > 0, "trace width cannot be zero");
+    ) -> Result<ParsedOodFrame<E>, DeserializationError> {
+        assert!(main_trace_width > 0, "trace width cannot be zero");
         assert!(num_evaluations > 0, "number of evaluations cannot be zero");
 
-        let mut reader = SliceReader::new(&self.trace_at_z1);
-        let current = E::read_batch_from(&mut reader, trace_width)?;
+        // parse current and next trace states for main and auxiliary trace evaluation frames
+        let mut reader = SliceReader::new(&self.trace_states);
+        let current = E::read_batch_from(&mut reader, main_trace_width)?;
+        let current_aux = E::read_batch_from(&mut reader, aux_trace_width)?;
+        let next = E::read_batch_from(&mut reader, main_trace_width)?;
+        let next_aux = E::read_batch_from(&mut reader, aux_trace_width)?;
         if reader.has_more_bytes() {
             return Err(DeserializationError::UnconsumedBytes);
         }
 
-        let mut reader = SliceReader::new(&self.trace_at_z2);
-        let next = E::read_batch_from(&mut reader, trace_width)?;
-        if reader.has_more_bytes() {
-            return Err(DeserializationError::UnconsumedBytes);
-        }
+        // instantiate the frames from the parsed rows
+        let main_frame = EvaluationFrame::from_rows(current, next);
+        let aux_frame = if aux_trace_width > 0 {
+            Some(EvaluationFrame::from_rows(current_aux, next_aux))
+        } else {
+            None
+        };
 
+        // parse the constraint evaluations
         let mut reader = SliceReader::new(&self.evaluations);
         let evaluations = E::read_batch_from(&mut reader, num_evaluations)?;
         if reader.has_more_bytes() {
             return Err(DeserializationError::UnconsumedBytes);
         }
 
-        Ok((EvaluationFrame::from_rows(current, next), evaluations))
+        Ok((main_frame, aux_frame, evaluations))
     }
 }
 
 impl Serializable for OodFrame {
     /// Serializes `self` and writes the resulting bytes into the `target`.
     fn write_into<W: ByteWriter>(&self, target: &mut W) {
-        // write trace rows (both rows have the same number of bytes)
-        target.write_u16(self.trace_at_z1.len() as u16);
-        target.write_u8_slice(&self.trace_at_z1);
-        target.write_u8_slice(&self.trace_at_z2);
+        // write trace rows
+        target.write_u16(self.trace_states.len() as u16);
+        target.write_u8_slice(&self.trace_states);
 
         // write constraint evaluations row
         target.write_u16(self.evaluations.len() as u16);
@@ -149,17 +143,15 @@ impl Deserializable for OodFrame {
     /// Returns an error of a valid OOD frame could not be read from the specified `source`.
     fn read_from<R: ByteReader>(source: &mut R) -> Result<Self, DeserializationError> {
         // read trace rows
-        let trace_row_bytes = source.read_u16()? as usize;
-        let trace_at_z1 = source.read_u8_vec(trace_row_bytes)?;
-        let trace_at_z2 = source.read_u8_vec(trace_row_bytes)?;
+        let num_trace_state_bytes = source.read_u16()? as usize;
+        let trace_states = source.read_u8_vec(num_trace_state_bytes)?;
 
         // read constraint evaluations row
-        let constraint_row_bytes = source.read_u16()? as usize;
-        let evaluations = source.read_u8_vec(constraint_row_bytes)?;
+        let num_constraint_evaluation_bytes = source.read_u16()? as usize;
+        let evaluations = source.read_u8_vec(num_constraint_evaluation_bytes)?;
 
         Ok(OodFrame {
-            trace_at_z1,
-            trace_at_z2,
+            trace_states,
             evaluations,
         })
     }
